import enum


class Color(enum.Enum):
    RED = enum.auto()
    GREEN = enum.auto()
    YELLOW = enum.auto()
    BLUE = enum.auto()
    MAGENTA = enum.auto()
    CYAN = enum.auto()


def colorize(message: str, color: Color) -> str:
    color_codes = {
        Color.RED: "\033[1;31m",
        Color.GREEN: "\033[1;32m",
        Color.YELLOW: "\033[1;33m",
        Color.BLUE: "\033[1;34m",
        Color.MAGENTA: "\033[35m",
        Color.CYAN: "\033[1;36m",
    }
    color_end_code = "\033[0m"

    return color_codes[color] + message + color_end_code



from __future__ import annotations

import argparse
import sys

from pykeychain import Storage

from ._client import Client, ClientError, Item
from ._clipboard import write_to_clipboard
from ._colorizer import Color, colorize
from ._constants import PACKAGE_NAME, SERVICE_NAME


def _parce_cli_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    subparser = parser.add_subparsers(dest="command", help="Command")

    parser_get = subparser.add_parser("get", help="Get one time password")
    parser_get.add_argument("account", type=str, nargs="?", default="", help="Account name")

    parser_add = subparser.add_parser("add", help="Add secret for TOTP")
    parser_add.add_argument("account", type=str, help="Account name")
    parser_add.add_argument("secret", type=str, help="Secret")

    parser_delete = subparser.add_parser("delete", help="Delete entry")
    parser_delete.add_argument("account", type=str, help="Account name")

    parser_search = subparser.add_parser("search", help="Search for items")
    parser_search.add_argument("pattern", type=str, help="search pattern")

    parser.add_argument("--version", action="store_true", default=False)

    return parser.parse_args()


def print_items(items: list[Item]) -> None:
    for item in items:
        print(f"{item.account}: {item.otp}")


def print_error(message: str) -> None:
    print(colorize(message, Color.RED))


def print_version() -> None:
    import pkg_resources

    version = pkg_resources.get_distribution(PACKAGE_NAME).version
    print(version)


def entrypoint() -> None:
    args = _parce_cli_arguments()

    if args.version:
        print_version()
        sys.exit(0)

    storage = Storage(SERVICE_NAME)
    client = Client(storage)

    try:
        if args.command == "get":
            items = client.get_otp(args.account)
            print_items(items)
            write_to_clipboard(items[0].otp)

        elif args.command == "add":
            client.set_secret(args.account, args.secret)

        elif args.command == "delete":
            reply = input(f"Are you sure want to delete TOTP secret for {args.account}? (Yy/Nn): ")
            if reply in ["Y", "y"]:
                client.delete_secret(args.account)

        elif args.command == "search":
            items = client.search_items(args.pattern)
            if not items:
                print_error(f"Nothing found for search pattern {args.pattern}")
            else:
                print_items(items)
        else:
            print_error("Missing required argument 'command'")
            sys.exit(4)

    except ClientError as e:
        print_error(str(e))
        sys.exit(e.return_code)


if __name__ == "__main__":
    entrypoint()



from __future__ import annotations

import binascii
import dataclasses

import pyotp
from pykeychain import AlreadyExistsException, NotFoundException, Storage


class ClientError(Exception):
    def __init__(self, message: str, return_code: int) -> None:
        self.message = message
        self.return_code = return_code

    def __str__(self) -> str:
        return self.message


@dataclasses.dataclass
class Item:
    account: str
    otp: str


class Client:
    def __init__(self, storage: Storage):
        self.storage = storage

    def _get_otp_from_secret(self, secret: str) -> str:
        totp = pyotp.TOTP(secret)
        return totp.now()

    def search_items(self, pattern: str) -> list[Item]:
        if not pattern:
            accounts = self.storage.get_all_accounts()
        else:
            accounts = self.storage.search_accounts(pattern)
        if not accounts:
            return []

        items = []
        for account in accounts:
            secret = self.storage.get_password(account)
            otp = self._get_otp_from_secret(secret)
            item = Item(account=account, otp=otp)
            items.append(item)
        return items

    def get_otp(self, account: str) -> list[Item]:
        try:
            secret = self.storage.get_password(account)
        except NotFoundException:
            items = self.search_items(account)
            if not items:
                raise ClientError(message=f"Entry {account} not found.", return_code=1)
            return items

        otp = self._get_otp_from_secret(secret)
        return [Item(account=account, otp=otp)]

    def set_secret(self, account: str, secret: str) -> None:
        try:
            pyotp.TOTP(secret).now()
        except binascii.Error as e:
            raise ClientError(message=f"Invalid TOTP secret. {e}", return_code=3)

        try:
            self.storage.save_password(account, secret)
        except AlreadyExistsException:
            raise ClientError(message=f"Entry {account} already exists", return_code=2)

    def delete_secret(self, account: str) -> None:
        try:
            self.storage.delete(account)
        except NotFoundException:
            raise ClientError(message=f"Entry {account} not found", return_code=1)



import subprocess  # nosec


def write_to_clipboard(output: str) -> None:
    process = subprocess.Popen("pbcopy", env={"LANG": "en_US.UTF-8"}, stdin=subprocess.PIPE)  # nosec
    process.communicate(output.encode("utf-8"))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytotp_client']

package_data = \
{'': ['*']}

install_requires = \
['pykeychain>=1.1.0,<2.0.0', 'pyotp>=2.7.0,<3.0.0']

entry_points = \
{'console_scripts': ['totp = pytotp_client:entrypoint']}

setup_kwargs = {
    'name': 'pytotp-client',
    'version': '0.3.0',
    'description': '',
    'long_description': '# TOTP client for macOS\n\n- Written in Python\n- Secrets are stored in macOS keychain.\n\n## How to work with pytotp_client\n\nAdd secret to keychain\n\n```shell\ntotp add google.com BBBBDDDD\n```\n\nGet one time password\n\n```shell\ntotp get google.com\n```\n\nRemove secret from keychain\n\n```shell\ntotp delete google.com\n```\n\n## Where is my secrets stored?\n\nIn macOS default keychain.\n - Name: pytotp_client\n - Type: application password',
    'author': 'Elisei',
    'author_email': 'elisey.rav@gmail.comp',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/elisey/pytotp_client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)
